// file      : xsd-frontend/semantic-graph/complex.hxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_FRONTEND_SEMANTIC_GRAPH_COMPLEX_HXX
#define XSD_FRONTEND_SEMANTIC_GRAPH_COMPLEX_HXX

#include <xsd-frontend/semantic-graph/elements.hxx>

#include <cult/containers/vector.hxx>

namespace XSDFrontend
{
  namespace SemanticGraph
  {
    class Complex: public virtual Type, public virtual Scope
    {
      typedef
      Cult::Containers::Vector<Inherits*>
      InheritsList;

    public:
      typedef
      Bits::PointerIterator<InheritsList::ConstIterator>
      InheritsIterator;

      InheritsIterator
      inherits_begin () const
      {
        return inherits_.begin ();
      }

      InheritsIterator
      inherits_end () const
      {
        return inherits_.end ();
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Complex (Path const& file, UnsignedLong line, UnsignedLong column);
      Complex (); // For virtual inheritance.

      using Type::add_edge_right;
      using Scope::add_edge_left;

      Void
      add_edge_left (Inherits& e)
      {
        inherits_.push_back (&e);
      }

    private:
      InheritsList inherits_;
    };
  }
}

#endif  // XSD_FRONTEND_SEMANTIC_GRAPH_COMPLEX_HXX
